﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Boats
{
    public partial class FormMapWithSetBoats : Form
    {
        /// <summary>
        /// Объект от класса карты с набором объектов
        /// </summary>
        private MapWithSetBoatsGeneric<DrawingObjectBoat, AbstractMap> _mapBoatsCollectionGeneric;
        /// <summary>
        /// Конструктор
        /// </summary>
        public FormMapWithSetBoats()
        {
            InitializeComponent();
        }
        /// <summary>
        /// Выбор карты
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ComboBoxSelectorMap_SelectedIndexChanged(object sender, EventArgs e)
        {
            AbstractMap map = null;
            switch (comboBoxSelectorMap.Text)
            {
                case "Простая карта":
                    map = new SimpleMap();
                    break;
                case "Линии карта":
                    map = new LineMap();
                    break;
                case "Океан карта":
                    map = new OceanMap();
                    break;
            }
            if (map != null)
            {
                _mapBoatsCollectionGeneric = new MapWithSetBoatsGeneric<DrawingObjectBoat, AbstractMap>(
                pictureBox.Width, pictureBox.Height, map);
            }
            else
            {
                _mapBoatsCollectionGeneric = null;
            }
        }
        /// <summary>
        /// Добавление объекта
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonAddBoat_Click(object sender, EventArgs e)
        {
            if (_mapBoatsCollectionGeneric == null)
            {
                return;
            }
            FormBoat form = new();
            if (form.ShowDialog() == DialogResult.OK)
            {
                bool added = false;
                if (form.SelectedBoat != null)
                {
                    DrawingObjectBoat boat = new(form.SelectedBoat);
                    if (_mapBoatsCollectionGeneric + boat != -1)
                    {
                        MessageBox.Show("Объект добавлен");
                        pictureBox.Image = _mapBoatsCollectionGeneric.ShowSet();
                        added = true;
                    }
                }
                if (!added)
                {
                    MessageBox.Show("Не удалось добавить объект");
                }
            }
        }
        /// <summary>
        /// Удаление объекта
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonRemoveBoat_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(maskedTextBoxPosition.Text))
            {
                return;
            }
            if (MessageBox.Show("Удалить объект?", "Удаление",
                MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            int pos = Convert.ToInt32(maskedTextBoxPosition.Text);
            pos -= 1;
            if (_mapBoatsCollectionGeneric - pos != null)
            {
                MessageBox.Show("Объект удален");
                pictureBox.Image = _mapBoatsCollectionGeneric.ShowSet();
            }
            else
            {
                MessageBox.Show("Не удалось удалить объект");
            }
        }
        /// <summary>
        /// Вывод набора
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonShowStorage_Click(object sender, EventArgs e)
        {
            if (_mapBoatsCollectionGeneric == null)
            {
                return;
            }
            pictureBox.Image = _mapBoatsCollectionGeneric.ShowSet();
        }
        /// <summary>
        /// Вывод карты
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonShowOnMap_Click(object sender, EventArgs e)
        {
            if (_mapBoatsCollectionGeneric == null)
            {
                return;
            }
            pictureBox.Image = _mapBoatsCollectionGeneric.ShowOnMap();
        }
        /// <summary>
        /// Перемещение
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonMove_Click(object sender, EventArgs e)
        {
            if (_mapBoatsCollectionGeneric == null)
            {
                return;
            }
            // Получаем имя кнопки
            string name = ((Button)sender)?.Name ?? string.Empty;
            Direction dir = Direction.None;
            switch (name)
            {
                case "ButtonUp":
                    dir = Direction.Up;
                    break;
                case "ButtonDown":
                    dir = Direction.Down;
                    break;
                case "ButtonLeft":
                    dir = Direction.Left;
                    break;
                case "ButtonRight":
                    dir = Direction.Right;
                    break;
            }
            pictureBox.Image = _mapBoatsCollectionGeneric.MoveObject(dir);
        }
    }
}
