﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Boats
{
    /// <summary>
    /// Параметризованный набор объектов
    /// </summary>
    /// <typeparam name="T"></typeparam>
    internal class SetBoatsGeneric<T>
        where T : class
    {
        /// <summary>
        /// Массив объектов, которые храним
        /// </summary>
        private readonly T[] _places;
        /// <summary>
        /// Количество объектов в массиве
        /// </summary>
        public int Count => _places.Length;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetBoatsGeneric(int count)
        {
            _places = new T[count];
        }
        /// <summary>
        /// Добавление объекта в набор
        /// </summary>
        /// <param name="boat">Добавляемая лодка</param>
        /// <returns></returns>
        public int Insert(T boat)
        {
            // Вставка в начало набора
            return Insert(boat, 0);
        }
        /// <summary>
        /// Добавление объекта в набор на конкретную позицию
        /// </summary>
        /// <param name="boat">Добавляемая лодка</param>
        /// <param name="position">Позиция</param>
        /// <returns></returns>
        public int Insert(T boat, int position)
        {
            // Проверка позиции
            if (position < 0 || position >= _places.Length)
                return -1;
            // Проверка, что элемент массива по этой позиции пустой
            if (_places[position] != null)
            {
                // Если нет, проверим, что после вставляемого элемента в массиве есть пустой элемент
                int i = position + 1;
                int nullIndex = -1;
                while (i < _places.Length)
                {
                    if (_places[i] == null)
                    {
                        nullIndex = i;
                        break;
                    }
                    i++;
                }
                // Если свободной нет, то выходим
                if (nullIndex < 0)
                {
                    return -1;
                }
                else
                {
                    // Если есть, сдвигаем все объекты, находящиеся
                    // справа от позиции до первого пустого элемента
                    i = nullIndex - 1;
                    while (i >= position)
                    {
                        _places[i + 1] = _places[i];
                        i--;
                    }
                }
            }
            // Вставка по позиции
            _places[position] = boat;
            return position;
        }
        /// <summary>
        /// Удаление объекта из набора с конкретной позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns>Возвращает удаляемый объект или null, если не удалось удалить</returns>
        public T Remove(int position)
        {
            // Проверка позиции
            if (position < 0 || position >= _places.Length)
                return null;
            if (_places[position] == null)
            {
                return null;
            }
            // Удаление объекта из массива, присовив элементу массива значение null
            T boat = _places[position];
            _places[position] = null;
            return boat;
        }
        /// <summary>
        /// Получение объекта из набора по позиции
        /// </summary>
        /// <param name="position"></param>
        /// <returns>Возвращает объект по позиции</returns>
        public T Get(int position)
        {
            // Проверка позиции
            if (position < 0 || position >= _places.Length)
                return null;
            return _places[position];
        }
    }
}
